"""Package for management procedures as described in KNX-Standard 3.5.2."""
from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from xknx.exceptions import ManagementConnectionRefused, ManagementConnectionTimeout
from xknx.telegram import Telegram, apci, tpci
from xknx.telegram.address import IndividualAddress, IndividualAddressableType

if TYPE_CHECKING:
    from xknx import XKNX

logger = logging.getLogger("xknx.management.procedures")


async def dm_restart(xknx: XKNX, individual_address: IndividualAddressableType) -> None:
    """Restart the device."""
    async with xknx.management.connection(
        address=IndividualAddress(individual_address)
    ) as connection:
        logger.debug("Requesting a Basic Restart of %s.", individual_address)
        # A_Restart will not be ACKed by the device, so it is manually sent to avoid timeout and retry
        seq_num = next(connection.sequence_number)
        telegram = Telegram(
            destination_address=connection.address,
            source_address=xknx.current_address,
            payload=apci.Restart(),
            tpci=tpci.TDataConnected(sequence_number=seq_num),
        )
        await xknx.cemi_handler.send_telegram(telegram)


async def nm_individual_address_check(
    xknx: XKNX, individual_address: IndividualAddressableType
) -> bool:
    """Check if the individual address is occupied on the network."""
    try:
        async with xknx.management.connection(
            address=IndividualAddress(individual_address)
        ) as connection:
            try:
                response = await connection.request(
                    payload=apci.DeviceDescriptorRead(descriptor=0),
                    expected=apci.DeviceDescriptorResponse,
                )

            except ManagementConnectionTimeout as ex:
                # if nothing is received (-> timeout) IA is free
                logger.debug("No device answered to connection attempt. %s", ex)
                return False
            if isinstance(response.payload, apci.DeviceDescriptorResponse):
                # if response is received IA is occupied
                logger.debug("Device found at %s", individual_address)
                return True
            return False
    except ManagementConnectionRefused as ex:
        # if Disconnect is received immediately, IA is occupied
        logger.debug("Device does not support transport layer connections. %s", ex)
        return True



"""
Module for managing a remote temperature value.

DPT 9.001.
"""
from __future__ import annotations

from xknx.dpt import DPTArray, DPTBinary, DPTTemperature

from .remote_value import RemoteValue


class RemoteValueTemp(RemoteValue[float]):
    """Abstraction for remote value of KNX 9.001 (DPT_Value_Temp)."""

    def to_knx(self, value: float) -> DPTArray:
        """Convert value to payload."""
        return DPTTemperature.to_knx(value)

    def from_knx(self, payload: DPTArray | DPTBinary) -> float:
        """Convert current payload to value."""
        return DPTTemperature.from_knx(payload)



"""
This package contains all objects for connecting to sockets.
"""
# flake8: noqa
from .ip_transport import KNXIPTransport
from .tcp_transport import TCPTransport
from .udp_transport import UDPTransport



"""
Module for managing a DTP Scene Number remote value.

DPT 17.001.
"""
from __future__ import annotations

from xknx.dpt import DPTArray, DPTBinary, DPTSceneNumber

from .remote_value import RemoteValue


class RemoteValueSceneNumber(RemoteValue[int]):
    """Abstraction for remote value of KNX DPT 17.001 (DPT_Scene_Number)."""

    def to_knx(self, value: int) -> DPTArray:
        """Convert value to payload."""
        return DPTSceneNumber.to_knx(value)

    def from_knx(self, payload: DPTArray | DPTBinary) -> int:
        """Convert current payload to value."""
        return DPTSceneNumber.from_knx(payload)
